/* This is the wrapper program for LyX/Cygwin. Using this
 * wrapper program no DOS window will be present running LyX.
 * The bad side of this: no error output could be seen ;-)
 *
 * It launches the real binary program using the native Windows GUI
 * lyx-qtwin.exe, unless it is called lyx-x11.exe in which case
 * lyx-xforms.exe gets started.
 *
 * compile this source with following options set:
 *
 *    gcc lyx-win.c -O2 -o lyx-win -static -Wall -Wno-format \
 *                   -Wstrict-prototypes -Wmissing-prototypes \
 *                   -mwindows -e _mainCRTStartup
 *
 * Claus Hentschel, 2002-01-17
 * Enrico Forestieri, 2007-02-15
 */
#include <stdio.h>            /* standard io library */
#include <stdlib.h>           /* standard library */
#include <unistd.h>           /* sleep */
#include <process.h>          /* spawnv */
#include <ctype.h>            /* isblank */
#include <string.h>           /* standard string library */
#include <limits.h>           /* PATH_MAX */
#include <sys/cygwin.h>       /* cygwin_conv_to_posix_path */
#include <errno.h>
#include <windows.h>

int RunningX(const char *);
const char *GetXname(void);

/* Returns non-zero if the Xwindow system is running */
/* Must use system(), as popen() throws a tantrum... */ 

int RunningX(const char *xname)
{
    FILE *fp;
    char buf[128];
    int n = 0;
    char *outname = tmpnam(NULL);
    sprintf(buf, "ps -efW | grep -i '%s' | wc -l > %s", xname, outname);
    system(buf);
    fp = fopen(outname, "r");
    if (fp) {
	fgets(buf, sizeof(buf), fp);
	fclose(fp);
	sscanf(buf, "%d", &n);
    }
    remove(outname);
    return(n);
}

/* Get the name of the X server */

const char *GetXname(void)
{
    static char buf[256];
    char *xname = NULL;
    FILE *fp = fopen("/etc/lyxprofile", "r");
    if (fp) {
	while (!xname && fgets(buf, sizeof(buf), fp)) {
	    char *head;
	    char *tail;
	    if (strncmp(buf, "XSERVERNAME", 11) == 0
		    && (head = strchr(buf, '='))) {
		++head;
		while (isblank(*head) || *head == '"')
		    ++head;
		tail = head + strlen(head) - 1;
		*tail-- = 0;
		while (isblank(*tail) || *tail == '"')
		    --tail;
		*(tail+1) = 0;
		if (strlen(head) > 0)
		    xname = head;
	    }
	}
	fclose(fp);
    }
    return xname;
}


int main (int argc, char **argv, char **environ)
{
    FILE *fp;
    char *s;
    char posixpath[PATH_MAX];         /* converted path */
    char cmd[4096] = "lyx-qtwin ";    /* user command */
    char const *nargs[5] = {          /* execute with login /bin/bash */
	    "/bin/bash", "--login",
	    "-c" , cmd ,
	    NULL
    };
    int i = 1; /* just to count */

    /* check if the X11 version should be launched */
    s = strrchr(argv[0],'-');
    if (s && (strcasecmp(s, "-x11") == 0 || strcasecmp(s, "-x11.exe") == 0))
    {
	/* Get the X server name from /etc/lyxprofile */
	const char *xname = GetXname();

	/* if the X server is not already running, try to start it through
	 * startxserver, otherwise bail out with the native version of LyX
	 */
	if (xname) {
	    if (strlen(xname) < 64) {
		if (RunningX(xname))
		    strcpy(cmd, "lyx-xforms ");
		else
		{
		    system("/usr/local/bin/startxserver");
		    sleep(1);
		    if (RunningX(xname))
			strcpy(cmd, "lyx-xforms ");
		}
	    } else
		MessageBox(0, "Please, shorten XSERVERNAME in /etc/lyxprofile",
			"ERROR: X server name is too long", 0);
	} else
	    MessageBox(0, "Please, set XSERVERNAME in /etc/lyxprofile",
		    "ERROR: Cannot find X server name", 0);
    }

    /* do for all "real" args */
    while (i < argc)
    {
	int done = 0;
	int lyxfile = (s = strrchr(argv[i], '.')) && strcasecmp(s, ".lyx") == 0;
	strcat(cmd, "\""); /* add quote before */
	cygwin_conv_to_posix_path(argv[i], posixpath) ;
	/* Hack to account for shares */
	if (lyxfile && argv[i][0] == '\\' && argv[i][1] != '\\')
	    strcat(cmd, "/");
	strcat(cmd, posixpath); /* add the argument */
	strcat(cmd, "\" "); /* add closing quote */
	if (!done && lyxfile && (s = strrchr(posixpath,'/')))
	{
	    done = 1;
	    fp = fopen("/tmp/cdpath","w");
	    if (fp)
	    {
		*s = 0;
		fprintf(fp, "%s\n", posixpath);
		fclose(fp);
	    }
	}
	++i;
    }

    strcat(cmd, "</dev/null 1>/dev/null 2>&1");

    /* fprintf(stderr , "Command is: |%s|\n", cmd); */
    /* ensure bash reads our global env changes */
    putenv("BASH_ENV=/etc/lyxprofile") ;
    spawnv(_P_NOWAIT, "/bin/bash", nargs);  /* exec sub command */
    return(0) ; /* exit with no error */
}
